import unittest
import casadi.casadi as cs
import opengen as og


class ProblemBuilderTestCase(unittest.TestCase):

    def test_add_sx_decision_variables(self):
        u = cs.SX.sym("u", 10)
        x = cs.SX.sym("x", 4)
        pb = og.builder.ProblemBuilder()
        pb.add_decision_variable(u, x)
        self.assertEqual(14, len(pb.decision_variables))

    def test_add_mx_decision_variables(self):
        u = cs.MX.sym("u", 10)
        x = cs.MX.sym("x", 4)
        pb = og.builder.ProblemBuilder()
        pb.add_decision_variable(u, x)
        self.assertEqual(2, len(pb.decision_variables))


if __name__ == '__main__':
    unittest.main()



from ..constraints.rectangle import Rectangle
from ..constraints.ball_inf import BallInf


class SetYCalculator:

    LARGE_NUM = 1e12

    def __init__(self, set_c):
        self.__set_c = set_c

    def __obtain_y_with_c_compact(self):
        return BallInf(None, SetYCalculator.LARGE_NUM)

    def __obtain_y_with_c_rectangle(self):
        c = self.__set_c
        xmin = c.xmin
        xmax = c.xmax
        if xmin is not None:
            n = len(xmin)
        elif xmax is not None:
            n = len(xmax)
        else:
            raise Exception("Fatal error: both xmin and xmax are None")

        if xmin is None:
            ymin = [0.0] * n
        else:
            ymin = [-SetYCalculator.LARGE_NUM] * n
            for i in range(n):
                if xmin[i] == float('-inf'):
                    ymin[i] = 0.0

        if xmax is None:
            ymax = [0.0] * n
        else:
            ymax = [SetYCalculator.LARGE_NUM] * n
            for i in range(n):
                if xmax[i] == float('inf'):
                    ymax[i] = 0.0

        return Rectangle(ymin, ymax)

    def obtain(self):
        if isinstance(self.__set_c, Rectangle):
            return self.__obtain_y_with_c_rectangle()
        if self.__set_c.is_compact():
            return self.__obtain_y_with_c_compact()
        raise NotImplementedError()



from . import constraint
from typing import List


class CartesianProduct(constraint.Constraint):

    def __init__(self, dimension: int, segments: List[int], constraints: List[constraint.Constraint]):
        # TODO: Check whether input arguments are valid (types, ranges, etc)
        self.__dimension = dimension
        self.__segments = segments
        self.__constraints = constraints

    @property
    def constraints(self):
        return self.__constraints

    @property
    def segments(self):
        return self.__segments

    def segment_dimension(self, i):
        if i == 0:
            return self.__segments[0] + 1
        return self.__segments[i] - self.__segments[i-1]

    def distance_squared(self, u):
        squared_distance = 0.0
        num_segments = len(self.__segments)
        idx_previous = -1
        for i in range(num_segments):
            idx_current = self.__segments[i]
            ui = u[idx_previous+1:idx_current+1]
            current_sq_dist = self.__constraints[i].distance_squared(ui)
            idx_previous = idx_current
            squared_distance += current_sq_dist

        return squared_distance

    def project(self, u):
        raise NotImplementedError()

    def is_convex(self):
        flag = True
        for c in self.__constraints:
            flag &= c.is_convex()
        return flag

    def is_compact(self):
        for set_i in self.__constraints:
            if not set_i.is_compact():
                return False
        return True



import opengen as og
import casadi.casadi as cs

u = cs.SX.sym("u", 5)
p = cs.SX.sym("p", 2)
phi = og.functions.rosenbrock(u, p)
c = cs.vertcat(1.5 * u[0] - u[1],
               cs.fmax(0.0, u[2] - u[3] + 0.1))
bounds = og.constraints.Ball2(None, 1.5)
problem = og.builder.Problem(u, p, phi) \
    .with_penalty_constraints(c)        \
    .with_constraints(bounds)
meta = og.config.OptimizerMeta()               \
    .with_optimizer_name("potato")             \
    .with_version('0.1.2')                     \
    .with_licence('LGPLv3')
ros_config = og.config.RosConfiguration()      \
    .with_package_name("potato_optimizer")  \
    .with_node_name("potato_controller")    \
    .with_rate(35)                          \
    .with_description("cool ROS node")
local_open = '/home/chung/NetBeansProjects/RUST/optimization-engine/'
build_config = og.config.BuildConfiguration()  \
    .with_build_directory("my_optimizers")     \
    .with_build_mode(og.config.BuildConfiguration.RELEASE_MODE)  \
    .with_open_version(local_path=local_open) \
    .with_tcp_interface_config() \
    .with_ros(ros_config)
solver_config = og.config.SolverConfiguration()    \
    .with_tolerance(1e-5)                          \
    .with_delta_tolerance(1e-4)                    \
    .with_initial_penalty(890)                     \
    .with_penalty_weight_update_factor(5)

builder = og.builder.OpEnOptimizerBuilder(problem,
                                          meta,
                                          build_config,
                                          solver_config).with_verbosity_level(3)
builder.build()


o = og.tcp.OptimizerTcpManager('my_optimizers/potato')
o.start()
r = o.call([1.0, 50.0])
if r.is_ok():
    status = r.get()
    print(status.solution)
o.kill()
